/***************************************************************************
*   Copyright (C) 2004 by moty rahamim                                    *
*   moty@onexige.com                                                      *
*                                                                         *
*   just another metalic like Kstyle & Kwin, this theme is based on:      *
*   Phase & example by David Johnson                                      *
*   liquid by Daniel M. Duley a.k.a Mosfet <mosfet@idunno>                *
*   baghira by Thomas Lbking <thomas.luebking@web.de>                    * 
*                                                                         *
*                                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include <kconfig.h>
#include <kdeversion.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <klocale.h>

#include <qbitmap.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtooltip.h>

#include "Metal4kdeclient.h"
#include "embededData.h"

using namespace Metal4kde;

// global constants

static const int BUTTONSIZE = 25;
static const int DECOSIZE = 8;
static const int TITLESIZE = 26;
static const int FRAMESIZE = 4;
static const int SIDEFRAMESIZE = 3;

//////////////////////////////////////////////////////////////////////////////
// Metal4kdeFactory Class                                                     //
//////////////////////////////////////////////////////////////////////////////

bool Metal4kdeFactory::initialized_ = false;
Qt::AlignmentFlags Metal4kdeFactory::titlealign_ = Qt::AlignHCenter;
bool Metal4kdeFactory::enableFontShadow_;
bool Metal4kdeFactory::enableBubble_;

// window button decorations
QPixmap Metal4kdeFactory::brushedMetalPix_;
QPixmap Metal4kdeFactory::brushedMetalLightningPix_;
QPixmap Metal4kdeFactory::bubbleLPix_;
QPixmap Metal4kdeFactory::bubbleMPix_;
QPixmap Metal4kdeFactory::bubbleRPix_;
QPixmap Metal4kdeFactory::bubbleInactiveLPix_;
QPixmap Metal4kdeFactory::bubbleInactiveMPix_;
QPixmap Metal4kdeFactory::bubbleInactiveRPix_;
QPixmap Metal4kdeFactory::bubbleBevelDownLPix_;
QPixmap Metal4kdeFactory::bubbleBevelDownMPix_;
QPixmap Metal4kdeFactory::bubbleBevelDownRPix_;
QPixmap Metal4kdeFactory::frameLPix_;
QPixmap Metal4kdeFactory::frameRPix_;
QPixmap Metal4kdeFactory::lightningPix_;
QPixmap Metal4kdeFactory::shadingPix_;
QPixmap Metal4kdeFactory::buttonPix_;
QPixmap Metal4kdeFactory::buttonBevelDownPix_;
QPixmap Metal4kdeFactory::buttonBevelUpPix_;
QPixmap Metal4kdeFactory::buttonMaxPix_ ;
QPixmap Metal4kdeFactory::buttonMaxHoverPix_ ;
QPixmap Metal4kdeFactory::buttonMinPix_ ;
QPixmap Metal4kdeFactory::buttonMinHoverPix_ ;
QPixmap Metal4kdeFactory::buttonClosePix_ ;
QPixmap Metal4kdeFactory::buttonCloseHoverPix_ ;
QPixmap Metal4kdeFactory::buttonMenuPix_ ;
QPixmap Metal4kdeFactory::buttonMenuHoverPix_ ;
QPixmap Metal4kdeFactory::buttonStickyPix_ ;
QPixmap Metal4kdeFactory::buttonStickyHoverPix_ ;
QPixmap Metal4kdeFactory::buttonAbovePix_ ;
QPixmap Metal4kdeFactory::buttonAboveHoverPix_ ;
QPixmap Metal4kdeFactory::buttonBelowPix_ ;
QPixmap Metal4kdeFactory::buttonBelowHoverPix_ ;
QPixmap Metal4kdeFactory::buttonShadePix_ ;
QPixmap Metal4kdeFactory::buttonShadeHoverPix_ ;
QPixmap Metal4kdeFactory::buttonInactivePix_ ;
QPixmap Metal4kdeFactory::buttonInactiveHoverPix_ ;
QPixmap Metal4kdeFactory::buttonHelpPix_;
QPixmap Metal4kdeFactory::buttonHelpHoverPix_;
QPixmap Metal4kdeFactory::buttonHelpInactivePix_;
QPixmap Metal4kdeFactory::buttonHelpBevelPix_;

extern "C" KDecorationFactory* create_factory()
{
    return new Metal4kde::Metal4kdeFactory();
}

//////////////////////////////////////////////////////////////////////////////
// Metal4kdeFactory()
// ----------------
// Constructor

Metal4kdeFactory::Metal4kdeFactory()
{
    iMatrix_.rotate( 180.0 );
    buttonHelpCol_ = Qt::gray;
    buttonMaxCol_ = Qt::gray;
    buttonMinCol_ = Qt::gray;
    buttonCloseCol_ = Qt::gray;
    buttonMenuCol_ = Qt::gray;
    buttonStikyCol_ = Qt::gray;
    buttonAboveCol_ = Qt::gray;
    buttonBelowCol_ = Qt::gray;
    buttonShadeCol_ = Qt::gray;
    bubbleCol_ = Qt::gray;
    buttonInactiveCol_ = Qt::gray;
    brushedMetalCol_ = Qt::gray;


    readConfig();
    createPixmaps();
    initialized_ = true;
}

//////////////////////////////////////////////////////////////////////////////
// ~Metal4kdeFactory()
// -----------------
// Destructor

Metal4kdeFactory::~Metal4kdeFactory() { initialized_ = false; }

//////////////////////////////////////////////////////////////////////////////
// adjustHSV()
// -----------
//
QPixmap* Metal4kdeFactory::adjustHSV( QImage &img, const QColor &c, bool blend ) const
{
#ifdef BACKTRACE
    qWarning( "adjustHSV()" );
#endif

    QImage * tmp = new QImage();
    tmp = adjustHSVImage( img, c, blend );
    QPixmap *pix = new QPixmap;
    pix->convertFromImage( *tmp );
    delete tmp;
    return ( pix );
}

//////////////////////////////////////////////////////////////////////////////
// adjustHSVImage()
// -----------
//
QImage* Metal4kdeFactory::adjustHSVImage( QImage &img, const QColor &c, bool blend ) const
{
#ifdef BACKTRACE
    qWarning( "adjustHSVImage()" );
#endif

    QColor bgColor = Qt::red; //optionsHandler_->customColors_[CustomWidgetBorderColor];
    int colorShift = 50; //optionsHandler_->coloringAmount_;
    if ( img.width() == 1024 && img.height() == 100 )
    {
        colorShift = 20;
    }

    img = img.convertDepth( 32 );
    QImage *dest = new QImage( img.width(), img.height(), 32 );
    dest->setAlphaBuffer( true );
    unsigned int *data = ( unsigned int * ) img.bits();
    unsigned int *destData = ( unsigned int* ) dest->bits();
    int total = img.width() * img.height();
    int current;
    int delta;
    int destR, destG, destB, alpha;
    int srcR = c.red() + colorShift;
    int srcG = c.green() + colorShift;
    int srcB = c.blue() + colorShift;
    float srcPercent, destPercent;
    for ( current = 0; current < total; ++current )
    {
        alpha = qAlpha( data[ current ] );
        delta = 255 - qRed( data[ current ] );
        destR = srcR - delta;
        destG = srcG - delta;
        destB = srcB - delta;
        if ( destR < 0 )
            destR = 0;
        if ( destG < 0 )
            destG = 0;
        if ( destB < 0 )
            destB = 0;
        if ( destR > 255 )
            destR = 255;
        if ( destG > 255 )
            destG = 255;
        if ( destB > 255 )
            destB = 255;

        if ( blend && alpha != 255 && alpha != 0 )
        {
            srcPercent = ( ( float ) alpha ) / 255.0;
            destPercent = 1.0 - srcPercent;
            destR = ( int ) ( ( srcPercent * destR ) + ( destPercent * bgColor.red() ) );
            destG = ( int ) ( ( srcPercent * destG ) + ( destPercent * bgColor.green() ) );
            destB = ( int ) ( ( srcPercent * destB ) + ( destPercent * bgColor.blue() ) );
            alpha = 255;
        }
        destData[ current ] = qRgba( destR, destG, destB, alpha );
    }
    return ( dest );
}


//////////////////////////////////////////////////////////////////////////////
// createPixmaps()
// -----------
//
void Metal4kdeFactory::createPixmaps()
{
    QColor a( 180, 208, 176 );
    QColor b( 239, 239, 239 );
    //QColor b(Qt::red);
    QImage *brushedMetalImg = new QImage( uic_findImage( "brushedMetal" ) );
    if ( brushedMetalImg->depth() < 32 )
        * brushedMetalImg = brushedMetalImg->convertDepth( 32 );
    Metal4kdeFactory::brushedMetalPix_ = *( adjustHSV( *brushedMetalImg , brushedMetalCol_, FALSE ) );
    delete brushedMetalImg;

    QImage *brushedMetaLightninglImg = new QImage( uic_findImage( "light" ) );
    if ( brushedMetaLightninglImg->depth() < 32 )
        * brushedMetaLightninglImg = brushedMetaLightninglImg->convertDepth( 32 );
    brushedMetalLightningPix_ = ( QPixmap ) * brushedMetaLightninglImg;
    delete brushedMetaLightninglImg;

    QImage *bubbleLImg = new QImage( uic_findImage( "bubbleL" ) );
    if ( bubbleLImg->depth() < 32 )
        * bubbleLImg = bubbleLImg->convertDepth( 32 );
    Metal4kdeFactory::bubbleLPix_ = *( adjustHSV( *bubbleLImg , bubbleCol_, FALSE ) );
    Metal4kdeFactory::bubbleInactiveLPix_ = *( adjustHSV( *bubbleLImg , bubbleInactiveCol_, FALSE ) );
    delete bubbleLImg;

    QImage *bubbleMImg = new QImage( uic_findImage( "bubbleM" ) );
    if ( bubbleMImg->depth() < 32 )
        * bubbleMImg = bubbleMImg->convertDepth( 32 );
    Metal4kdeFactory::bubbleMPix_ = *( adjustHSV( *bubbleMImg , bubbleCol_, FALSE ) );
    Metal4kdeFactory::bubbleInactiveMPix_ = *( adjustHSV( *bubbleMImg , bubbleInactiveCol_, FALSE ) );
    delete bubbleMImg;

    QImage *bubbleRImg = new QImage( uic_findImage( "bubbleR" ) );
    if ( bubbleRImg->depth() < 32 )
        * bubbleRImg = bubbleRImg->convertDepth( 32 );
    Metal4kdeFactory::bubbleRPix_ = *( adjustHSV( *bubbleRImg , bubbleCol_, FALSE ) );
    Metal4kdeFactory::bubbleInactiveRPix_ = *( adjustHSV( *bubbleRImg , bubbleInactiveCol_, FALSE ) );
    delete bubbleRImg;

    QImage *bubbleBevelDownLImg = new QImage( uic_findImage( "bubbleBevelDownL" ) );
    if ( bubbleBevelDownLImg->depth() < 32 )
        * bubbleBevelDownLImg = bubbleBevelDownLImg->convertDepth( 32 );
    bubbleBevelDownLPix_ = ( QPixmap ) * bubbleBevelDownLImg;
    delete bubbleBevelDownLImg;

    QImage *bubbleBevelDownMImg = new QImage( uic_findImage( "bubbleBevelDownM" ) );
    if ( bubbleBevelDownMImg->depth() < 32 )
        * bubbleBevelDownMImg = bubbleBevelDownMImg->convertDepth( 32 );
    bubbleBevelDownMPix_ = ( QPixmap ) * bubbleBevelDownMImg;
    delete bubbleBevelDownMImg;

    QImage *bubbleBevelDownRImg = new QImage( uic_findImage( "bubbleBevelDownR" ) );
    if ( bubbleBevelDownRImg->depth() < 32 )
        * bubbleBevelDownRImg = bubbleBevelDownRImg->convertDepth( 32 );
    bubbleBevelDownRPix_ = ( QPixmap ) * bubbleBevelDownRImg;
    delete bubbleBevelDownRImg;

    QImage *frameLImg_ = new QImage( uic_findImage( "frameL" ) );
    if ( frameLImg_->depth() < 32 )
        * frameLImg_ = frameLImg_->convertDepth( 32 );
    Metal4kdeFactory::frameLPix_ = *( adjustHSV( *frameLImg_ , brushedMetalCol_, FALSE ) );
    delete frameLImg_;

    QImage *frameRImg_ = new QImage( uic_findImage( "frameR" ) );
    if ( frameLImg_->depth() < 32 )
        * frameLImg_ = frameLImg_->convertDepth( 32 );
    Metal4kdeFactory::frameRPix_ = *( adjustHSV( *frameRImg_ , brushedMetalCol_, FALSE ) );
    delete frameLImg_;

    QImage *lightningImg = new QImage( uic_findImage( "lightning" ) );
    if ( lightningImg->depth() < 32 )
        * lightningImg = lightningImg->convertDepth( 32 );
    lightningPix_ = ( QPixmap ) * lightningImg;
    delete lightningImg;

    QImage *shadingImg = new QImage( uic_findImage( "shading" ) );
    if ( shadingImg->depth() < 32 )
        * shadingImg = shadingImg->convertDepth( 32 );
    shadingPix_ = ( QPixmap ) * shadingImg;
    delete shadingImg;

    QImage *buttonImg = new QImage( uic_findImage( "button" ) );
    if ( buttonImg->depth() < 32 )
        * buttonImg = buttonImg->convertDepth( 32 );

    Metal4kdeFactory::buttonPix_ = *( adjustHSV( *buttonImg , buttonBelowCol_, FALSE ) );

    Metal4kdeFactory::buttonMaxPix_ = *( adjustHSV( *buttonImg , buttonMaxCol_, FALSE ) );
    Metal4kdeFactory::buttonMaxHoverPix_ = *( adjustHSV( *buttonImg , buttonMaxCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonMinPix_ = *( adjustHSV( *buttonImg , buttonMinCol_, FALSE ) );
    Metal4kdeFactory::buttonMinHoverPix_ = *( adjustHSV( *buttonImg , buttonMinCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonClosePix_ = *( adjustHSV( *buttonImg , buttonCloseCol_, FALSE ) );
    Metal4kdeFactory::buttonCloseHoverPix_ = *( adjustHSV( *buttonImg , buttonCloseCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonStickyPix_ = *( adjustHSV( *buttonImg , buttonStikyCol_, FALSE ) );
    Metal4kdeFactory::buttonStickyHoverPix_ = *( adjustHSV( *buttonImg , buttonStikyCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonAbovePix_ = *( adjustHSV( *buttonImg , buttonAboveCol_, FALSE ) );
    Metal4kdeFactory::buttonAboveHoverPix_ = *( adjustHSV( *buttonImg , buttonAboveCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonBelowPix_ = *( adjustHSV( *buttonImg , buttonBelowCol_, FALSE ) );
    Metal4kdeFactory::buttonBelowHoverPix_ = *( adjustHSV( *buttonImg , buttonBelowCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonShadePix_ = *( adjustHSV( *buttonImg , buttonShadeCol_, FALSE ) );
    Metal4kdeFactory::buttonShadeHoverPix_ = *( adjustHSV( *buttonImg , buttonShadeCol_.light( 120 ), FALSE ) );
    Metal4kdeFactory::buttonInactivePix_ = *( adjustHSV( *buttonImg , buttonInactiveCol_, FALSE ) );
    Metal4kdeFactory::buttonInactiveHoverPix_ = *( adjustHSV( *buttonImg , buttonInactiveCol_.light( 120 ), FALSE ) );

    delete buttonImg;

    QImage *buttonBevelDownImg = new QImage( uic_findImage( "buttonShade" ) );
    if ( buttonBevelDownImg->depth() < 32 )
        * buttonBevelDownImg = buttonBevelDownImg->convertDepth( 32 );
    Metal4kdeFactory::buttonBevelDownPix_ = ( QPixmap ) * buttonBevelDownImg;;
    delete buttonBevelDownImg;

    Metal4kdeFactory::buttonBevelUpPix_ = Metal4kdeFactory::buttonBevelDownPix_.xForm( iMatrix_ );

    QImage *helpButtonImg = new QImage( uic_findImage( "helpButon" ) );
    if ( helpButtonImg->depth() < 32 )
        * helpButtonImg = helpButtonImg->convertDepth( 32 );
    Metal4kdeFactory::buttonHelpPix_ = *( adjustHSV( *helpButtonImg , buttonHelpCol_, FALSE ) );
    Metal4kdeFactory::buttonHelpHoverPix_ = *( adjustHSV( *helpButtonImg , buttonHelpCol_.light( 120 ), FALSE ) );
    delete helpButtonImg;

    QImage *helpButtonBevelImg = new QImage( uic_findImage( "helpButtonBevel" ) );
    if ( helpButtonBevelImg->depth() < 32 )
        * helpButtonBevelImg = helpButtonBevelImg->convertDepth( 32 );
    Metal4kdeFactory::buttonHelpBevelPix_ = ( QPixmap ) * helpButtonBevelImg;
    delete helpButtonBevelImg;
}
//////////////////////////////////////////////////////////////////////////////
// createDecoration()
// -----------------
// Create the decoration

KDecoration* Metal4kdeFactory::createDecoration( KDecorationBridge* b )
{
    return new Metal4kdeClient( b, this );
}

//////////////////////////////////////////////////////////////////////////////
// reset()
// -------
// Reset the handler. Returns true if decorations need to be remade, false if
// only a repaint is necessary

bool Metal4kdeFactory::reset( unsigned long changed )
{
    // read in the configuration
    initialized_ = false;
    bool confchange = readConfig();
    initialized_ = true;

    if ( confchange ||
            ( changed & ( SettingDecoration | SettingButtons | SettingBorder ) ) )
    {
        createPixmaps();
        return true;
    }
    else
    {
        resetDecorations( changed );
        return false;
    }
}

//////////////////////////////////////////////////////////////////////////////
// readConfig()
// ------------
// Read in the configuration file

bool Metal4kdeFactory::readConfig()
{
    // create a config object
    KConfig config( "kwinMetal4kderc" );
    config.setGroup( "General" );

    // grab settings
    Qt::AlignmentFlags oldalign = titlealign_;
    QString value = config.readEntry( "TitleAlignment", "AlignHCenter" );
    if ( value == "AlignLeft" ) titlealign_ = Qt::AlignLeft;
    else if ( value == "AlignHCenter" ) titlealign_ = Qt::AlignHCenter;
    else if ( value == "AlignRight" ) titlealign_ = Qt::AlignRight;

    bool oldEnableFontShadow = enableFontShadow_;
    enableFontShadow_ = config.readBoolEntry( "EnableFontsShadow", FALSE );
    bool oldEnableBubble = enableBubble_;
    enableBubble_ = config.readBoolEntry( "EnableBubble", FALSE );


    QColor oldButtonHelp = buttonHelpCol_;
    buttonHelpCol_.setRgb( config.readNumEntry( "ButtonHelp", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonMaxCol = buttonMaxCol_;
    buttonMaxCol_.setRgb( config.readNumEntry( "ButtonMax", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonMinCol = buttonMinCol_;
    buttonMinCol_.setRgb( config.readNumEntry( "ButtonMin", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonCloseCol = buttonCloseCol_;
    buttonCloseCol_.setRgb( config.readNumEntry( "ButtonClose", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonMenuCol = buttonMenuCol_;
    buttonMenuCol_.setRgb( config.readNumEntry( "ButtonMenu", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonStikyCol = buttonStikyCol_;
    buttonStikyCol_.setRgb( config.readNumEntry( "ButtonStiky", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonAboveCol = buttonAboveCol_;
    buttonAboveCol_.setRgb( config.readNumEntry( "ButtonAbove", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonBelowCol = buttonBelowCol_;
    buttonBelowCol_.setRgb( config.readNumEntry( "ButtonBelow", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonShadeCol = buttonShadeCol_;
    buttonShadeCol_.setRgb( config.readNumEntry( "ButtonShade", ( int ) Qt::gray.rgb() ) );
    QColor oldBubbleCol = bubbleCol_;
    bubbleCol_.setRgb( config.readNumEntry( "Bubble", ( int ) Qt::gray.rgb() ) );
    QColor oldBubbleInactiveCol = bubbleInactiveCol_;
    bubbleInactiveCol_.setRgb( config.readNumEntry( "BubbleInactive", ( int ) Qt::gray.rgb() ) );
    QColor oldButtonInactiveCol = buttonInactiveCol_;
    buttonInactiveCol_.setRgb( config.readNumEntry( "ButtunInactive", ( int ) Qt::gray.rgb() ) );
    QColor oldBrushedMetalCol = brushedMetalCol_;
    brushedMetalCol_.setRgb( config.readNumEntry( "BrushedMetal", ( int ) Qt::gray.rgb() ) );

    if ( oldalign == titlealign_ && oldButtonHelp == buttonHelpCol_ && oldButtonMaxCol == buttonMaxCol_ && oldButtonMinCol == buttonMinCol_ && oldButtonCloseCol == buttonCloseCol_ && oldButtonMenuCol == buttonMenuCol_ && oldButtonStikyCol == buttonStikyCol_ && oldButtonAboveCol == buttonAboveCol_ && oldButtonBelowCol == buttonBelowCol_ && oldButtonShadeCol == buttonShadeCol_ && oldBubbleCol == bubbleCol_ && oldBubbleInactiveCol == bubbleInactiveCol_ && oldButtonInactiveCol == buttonInactiveCol_ && oldBrushedMetalCol == brushedMetalCol_ && oldEnableFontShadow == enableFontShadow_ && oldEnableBubble == enableBubble_
       )
        return false;
    else
        return true;
}











//////////////////////////////////////////////////////////////////////////////
// Metal4kdeButton Class                                                      //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Metal4kdeButton()
// ---------------
// Constructor

Metal4kdeButton::Metal4kdeButton( Metal4kdeClient *parent, const char *name,
                                  const QString& tip, ButtonType type, QPixmap *pix, QPixmap *pixHover )
        : QButton( parent->widget(), name ), client_( parent ), type_( type ), pix_( 0 ), pixHover_( 0 ), lastmouse_( NoButton ), parent_( parent ), hover_( false )
{
    //setBackgroundMode(NoBackground);
    setBackgroundMode( PaletteBackground );
    //setBackgroundOrigin( WindowOrigin );
    setPaletteBackgroundPixmap( Metal4kdeFactory::brushedMetalPix_ );
    //setPaletteBackgroundColor(Qt::red);
    setFixedSize( BUTTONSIZE, BUTTONSIZE );
    setCursor( arrowCursor );
    /*    if ( type == ButtonClose || type == ButtonMin || type == ButtonMax || type == ButtonSticky || type == ButtonHelp )
            setPixmap( pix, pixHover );
        else */
    if ( pix )
        setPixmap( pix, pixHover );
    QToolTip::add( this, tip );

}

Metal4kdeButton::~Metal4kdeButton()
{
    if ( pix_ ) delete pix_;
    if ( pixHover_ ) delete pixHover_;
}

//////////////////////////////////////////////////////////////////////////////
// setBitmap()
// -----------
// Set the button decoration

// void Metal4kdeButton::setBitmap( const unsigned char *bitmap )
// {
//     if ( !bitmap ) return ; // no bitmap, probably the menu button
//
//     if ( deco_ ) delete deco_;
//     deco_ = new QBitmap( DECOSIZE, DECOSIZE, bitmap, true );
//     //deco_->setMask(*deco_);
//     //this->setMask(*deco_->mask());
//     repaint( false );
// }


//////////////////////////////////////////////////////////////////////////////
// setBitmap()
// -----------
// Set the button decoration

void Metal4kdeButton::setPixmap( QPixmap *pix, QPixmap *pixHover )
{
    if ( !pix ) return ; // no bitmap, probably the menu button

    if ( pix_ ) delete pix_;
    pix_ = new QPixmap( *pix );
    if ( pixHover_ ) delete pixHover_;
    pixHover_ = new QPixmap( *pixHover );
    //     pix_->setMask( *pix_->mask() );
    //     this->setMask( *pix_->mask() );
    repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// sizeHint()
// ----------
// Return size hint

QSize Metal4kdeButton::sizeHint() const
{
    return QSize( BUTTONSIZE, BUTTONSIZE );
}

//////////////////////////////////////////////////////////////////////////////
// enterEvent()
// ------------
// Mouse has entered the button

void Metal4kdeButton::enterEvent( QEvent *e )
{
    // if we wanted to do mouseovers, we would keep track of it here
    hover_ = true;
    QButton::enterEvent( e );
    repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// leaveEvent()
// ------------
// Mouse has left the button

void Metal4kdeButton::leaveEvent( QEvent *e )
{
    // if we wanted to do mouseovers, we would keep track of it here
    hover_ = false;
    QButton::leaveEvent( e );
    repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// mousePressEvent()
// -----------------
// Button has been pressed

void Metal4kdeButton::mousePressEvent( QMouseEvent* e )
{
    lastmouse_ = e->button();

    // translate and pass on mouse event
    int button = LeftButton;
    if ( ( type_ != ButtonMax ) && ( e->button() != LeftButton ) )
    {
        button = NoButton; // middle & right buttons inappropriate
    }
    QMouseEvent me( e->type(), e->pos(), e->globalPos(),
                    button, e->state() );
    QButton::mousePressEvent( &me );
}

//////////////////////////////////////////////////////////////////////////////
// mouseReleaseEvent()
// -----------------
// Button has been released

void Metal4kdeButton::mouseReleaseEvent( QMouseEvent* e )
{
    lastmouse_ = e->button();

    // translate and pass on mouse event
    int button = LeftButton;
    if ( ( type_ != ButtonMax ) && ( e->button() != LeftButton ) )
    {
        button = NoButton; // middle & right buttons inappropriate
    }
    QMouseEvent me( e->type(), e->pos(), e->globalPos(), button, e->state() );
    QButton::mouseReleaseEvent( &me );
}

//////////////////////////////////////////////////////////////////////////////
// drawButton()
// ------------
// Draw the button

void Metal4kdeButton::drawButton( QPainter *painter )
{
    if ( !Metal4kdeFactory::initialized() ) return ;

    int dx = 0, dy = 0;

    // paint a plain box with border
    QPoint pos( 0, 0 );
    painter->drawPixmap( 0 , 0, Metal4kdeFactory::brushedMetalLightningPix_, ( 512 - ( parent_->widget() ->width() / 2 ) + ( this->mapTo( topLevelWidget(), pos ).x() ) ), FRAMESIZE, BUTTONSIZE, BUTTONSIZE );


    if ( type_ == ButtonMenu )
    {

        // we paint the mini icon (which is 16 pixels high)
        dx = ( width() - 16 ) / 2;
        dy = ( height() - 16 ) / 2;
        if ( isDown() ) { dx++; dy++; }
        painter->drawPixmap( dx, dy, client_->icon().pixmap( QIconSet::Small,
                             QIconSet::Normal ) );
    }
    else if ( pix_ && pixHover_ )
    {
        if ( type_ == ButtonHelp )
            painter->drawPixmap( dx, dy, Metal4kdeFactory::buttonHelpBevelPix_ );
        else
            painter->drawPixmap( dx, dy, Metal4kdeFactory::buttonBevelDownPix_ );
    if ( isDown() ) { dx--; dy--; }
        painter->drawPixmap( dx, dy, hover_ ? *pixHover_ : *pix_ );
    }
}








//////////////////////////////////////////////////////////////////////////////
// Metal4kdeClient Class                                                      //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Metal4kdeClient()
// ---------------
// Constructor

Metal4kdeClient::Metal4kdeClient( KDecorationBridge *b, KDecorationFactory *f )
        : KDecoration( b, f ) { decoFactory_ = ( Metal4kdeFactory* ) f; }

Metal4kdeClient::~Metal4kdeClient()
{
    for ( int n = 0; n < ButtonTypeCount; n++ )
    {
        if ( button[ n ] ) delete button[ n ];
    }
}

//////////////////////////////////////////////////////////////////////////////
// init()
// ------
// Actual initializer for class

void Metal4kdeClient::init()
{
    createMainWidget( WResizeNoErase | WRepaintNoErase );
    widget() ->installEventFilter( this );

    // for flicker-free redraws
    widget() ->setBackgroundMode( NoBackground );
    //widget() ->setPaletteBackgroundPixmap( Metal4kdeFactory::brushedMetalPix_ );

    // setup layout
    QGridLayout *mainlayout = new QGridLayout( widget(), 4, 3 ); // 4x3 grid
    QHBoxLayout *titlelayout = new QHBoxLayout();
    titlebar_ = new QSpacerItem( 1, TITLESIZE, QSizePolicy::Expanding,
                                 QSizePolicy::Fixed );

    mainlayout->setResizeMode( QLayout::FreeResize );
    mainlayout->addRowSpacing( 0, FRAMESIZE ); // up
    mainlayout->addRowSpacing( 3, SIDEFRAMESIZE * 2 ); // down
    mainlayout->addColSpacing( 0, SIDEFRAMESIZE ); // left
    mainlayout->addColSpacing( 2, SIDEFRAMESIZE ); // right

    mainlayout->addLayout( titlelayout, 1, 1 );
    if ( isPreview() )
    {
        mainlayout->addWidget(
            new QLabel( i18n( "<b><center>Metal4kde preview</center></b>" ),
                        widget() ), 2, 1 );
    }
    else
    {
        mainlayout->addItem( new QSpacerItem( 0, 0 ), 2, 1 );
    }

    // the window should stretch
    mainlayout->setRowStretch( 2, 10 );
    mainlayout->setColStretch( 1, 10 );

    // setup titlebar buttons
    for ( int n = 0; n < ButtonTypeCount; n++ ) button[ n ] = 0;

    addButtons( titlelayout, options() ->titleButtonsLeft() );
    titlelayout->addItem( titlebar_ );
    addButtons( titlelayout, options() ->titleButtonsRight() );

    // connections
    connect( this, SIGNAL( keepAboveChanged( bool ) ), SLOT( keepAboveChange( bool ) ) );
    connect( this, SIGNAL( keepBelowChanged( bool ) ), SLOT( keepBelowChange( bool ) ) );
}

//////////////////////////////////////////////////////////////////////////////
// addButtons()
// ------------
// Add buttons to title layout

void Metal4kdeClient::addButtons( QBoxLayout *layout, const QString& s )
{
    const unsigned char * bitmap;
    QString tip;

    if ( s.length() > 0 )
    {
        for ( unsigned n = 0; n < s.length(); n++ )
        {
            switch ( s[ n ] )
            {
            case 'M':                                // Menu button
                if ( !button[ ButtonMenu ] )
                {
                    button[ ButtonMenu ] =
                        new Metal4kdeButton( this, "menu", i18n( "Menu" ),
                                             ButtonMenu, 0 );
                    connect( button[ ButtonMenu ], SIGNAL( pressed() ),
                             this, SLOT( menuButtonPressed() ) );
                    layout->addWidget( button[ ButtonMenu ] );

                }
                break;

            case 'S':                                // Sticky button
                if ( !button[ ButtonSticky ] )
                {
                    if ( isOnAllDesktops() )
                    {
                        //bitmap = stickydown_bits;
                        tip = i18n( "Un-Sticky" );
                    }
                    else
                    {
                        //bitmap = sticky_bits;
                        tip = i18n( "Sticky" );
                    }
                    button[ ButtonSticky ] =
                        new Metal4kdeButton( this, "sticky", tip,
                                             ButtonSticky, &Metal4kdeFactory::buttonStickyPix_, &Metal4kdeFactory::buttonStickyHoverPix_ );
                    connect( button[ ButtonSticky ], SIGNAL( clicked() ),
                             this, SLOT( toggleOnAllDesktops() ) );
                    layout->addWidget( button[ ButtonSticky ] );

                }
                break;

            case 'H':                                // Help button
                if ( ( !button[ ButtonHelp ] ) && providesContextHelp() )
                {
                    button[ ButtonHelp ] =
                        new Metal4kdeButton( this, "help", i18n( "Help" ),
                                             ButtonHelp, &Metal4kdeFactory::buttonHelpPix_, &Metal4kdeFactory::buttonHelpHoverPix_ );
                    connect( button[ ButtonHelp ], SIGNAL( clicked() ),
                             this, SLOT( showContextHelp() ) );
                    layout->addWidget( button[ ButtonHelp ] );
                }
                break;

            case 'I':                                // Minimize button
                if ( ( !button[ ButtonMin ] ) && isMinimizable() )
                {
                    button[ ButtonMin ] =
                        new Metal4kdeButton( this, "iconify", i18n( "Minimize" ),
                                             ButtonMin, &Metal4kdeFactory::buttonMinPix_, &Metal4kdeFactory::buttonMinHoverPix_ );
                    connect( button[ ButtonMin ], SIGNAL( clicked() ),
                             this, SLOT( minimize() ) );
                    layout->addWidget( button[ ButtonMin ] );
                }
                break;

            case 'A':                                // Maximize button
                if ( ( !button[ ButtonMax ] ) && isMaximizable() )
                {
                    if ( maximizeMode() == MaximizeFull )
                    {
                        //                        bitmap = minmax_bits;
                        tip = i18n( "Restore" );
                    }
                    else
                    {
                        //     bitmap = max_bits;
                        tip = i18n( "Maximize" );
                    }
                    button[ ButtonMax ] =
                        new Metal4kdeButton( this, "maximize", tip,
                                             ButtonMax, &Metal4kdeFactory::buttonMaxPix_, &Metal4kdeFactory::buttonMaxHoverPix_ );
                    connect( button[ ButtonMax ], SIGNAL( clicked() ),
                             this, SLOT( maxButtonPressed() ) );
                    layout->addWidget( button[ ButtonMax ] );
                }
                break;

            case 'X':                                // Close button
                if ( ( !button[ ButtonClose ] ) && isCloseable() )
                {
                    button[ ButtonClose ] =
                        new Metal4kdeButton( this, "close", i18n( "Close" ),
                                             ButtonClose, &Metal4kdeFactory::buttonClosePix_, &Metal4kdeFactory::buttonCloseHoverPix_ );
                    connect( button[ ButtonClose ], SIGNAL( clicked() ),
                             this, SLOT( closeWindow() ) );
                    layout->addWidget( button[ ButtonClose ] );
                }
                break;

            case 'F':                                // Above button
                if ( ( !button[ ButtonAbove ] ) )
                {
                    button[ ButtonAbove ] =
                        new Metal4kdeButton( this, "above",
                                             i18n( "Keep Above Others" ), ButtonAbove, &Metal4kdeFactory::buttonAbovePix_, &Metal4kdeFactory::buttonAboveHoverPix_ );
                    connect( button[ ButtonAbove ], SIGNAL( clicked() ),
                             this, SLOT( aboveButtonPressed() ) );
                    layout->addWidget( button[ ButtonAbove ] );
                }
                break;

            case 'B':                                // Below button
                if ( ( !button[ ButtonBelow ] ) )
                {
                    button[ ButtonBelow ] =
                        new Metal4kdeButton( this, "below",
                                             i18n( "Keep Below Others" ), ButtonBelow, &Metal4kdeFactory::buttonBelowPix_, &Metal4kdeFactory::buttonBelowHoverPix_ );
                    connect( button[ ButtonBelow ], SIGNAL( clicked() ),
                             this, SLOT( belowButtonPressed() ) );
                    layout->addWidget( button[ ButtonBelow ] );
                }
                break;

            case 'L':                                // Shade button
                if ( ( !button[ ButtonShade && isShadeable() ] ) )
                {
                    if ( isSetShade() )
                    {
                        //                        bitmap = shade_on_bits;
                        tip = i18n( "Unshade" );
                    }
                    else
                    {
                        //           bitmap = shade_off_bits;
                        tip = i18n( "Shade" );
                    }
                    button[ ButtonShade ] =
                        new Metal4kdeButton( this, "shade", tip,
                                             ButtonShade, &Metal4kdeFactory::buttonShadePix_, &Metal4kdeFactory::buttonShadeHoverPix_ );
                    connect( button[ ButtonShade ], SIGNAL( clicked() ),
                             this, SLOT( shadeButtonPressed() ) );
                    layout->addWidget( button[ ButtonShade ] );
                }
                break;

            case '_':                                // Spacer item
                layout->addSpacing( FRAMESIZE );
            }
        }
    }
}

//////////////////////////////////////////////////////////////////////////////
// doShape()
// ---------
// shape Corners

void Metal4kdeClient::doShape()
{
    QRegion mask( 0, 0, width(), height() );


    //  QRegion mask(width(), titlebar_->geometry().height());
    int r( width() );
    int b( height() - 1 );


    // Remove top-left corner.
    mask -= QRegion( 0, 0, 5, 1 );
    mask -= QRegion( 0, 1, 3, 1 );
    mask -= QRegion( 0, 2, 2, 1 );
    mask -= QRegion( 0, 3, 1, 2 );

    // Remove top-right corner.
    mask -= QRegion( r - 5, 0, 5, 1 );
    mask -= QRegion( r - 3, 1, 3, 1 );
    mask -= QRegion( r - 2, 2, 2, 1 );
    mask -= QRegion( r - 1, 3, 1, 2 );


    // Remove bottom-left corner.
    mask -= QRegion( 0, b - 0, 5, 1 );
    mask -= QRegion( 0, b - 1, 3, 1 );
    mask -= QRegion( 0, b - 2, 2, 1 );
    mask -= QRegion( 0, b - 4, 1, 2 );

    // Remove bottom-right corner.
    mask -= QRegion( r - 5, b - 0, 5, 1 );
    mask -= QRegion( r - 3, b - 1, 3, 1 );
    mask -= QRegion( r - 2, b - 2, 2, 1 );
    mask -= QRegion( r - 1, b - 4, 1, 2 );

    setMask( mask );
}


//////////////////////////////////////////////////////////////////////////////
// activeChange()
// --------------
// window active state has changed

void Metal4kdeClient::activeChange()
{
    for ( int n = 0; n < ButtonTypeCount; n++ )
        if ( button[ n ] ) button[ n ] ->reset();
    widget() ->repaint( false );
}

//////////////////////////////////////////////////////////////////////////////
// captionChange()
// ---------------
// The title has changed

void Metal4kdeClient::captionChange()
{
    widget() ->repaint( titlebar_->geometry(), false );
}

//////////////////////////////////////////////////////////////////////////////
// desktopChange()
// ---------------
// Called when desktop/sticky changes

void Metal4kdeClient::desktopChange()
{
    //     bool d = isOnAllDesktops();
    //     if ( button[ ButtonSticky ] )
    //     {
    //         button[ ButtonSticky ] ->setBitmap( d ? stickydown_bits : sticky_bits );
    //         QToolTip::remove( button[ ButtonSticky ] );
    //         QToolTip::add( button[ ButtonSticky ], d ? i18n( "Un-Sticky" ) : i18n( "Sticky" ) );
    //     }
}

//////////////////////////////////////////////////////////////////////////////
// iconChange()
// ------------
// The title has changed

void Metal4kdeClient::iconChange()
{
    //     if ( button[ ButtonMenu ] )
    //     {
    //         button[ ButtonMenu ] ->setBitmap( 0 );
    //         button[ ButtonMenu ] ->repaint( false );
    //     }
}

//////////////////////////////////////////////////////////////////////////////
// maximizeChange()
// ----------------
// Maximized state has changed

void Metal4kdeClient::maximizeChange()
{
    //     bool m = ( maximizeMode() == MaximizeFull );
    //     if ( button[ ButtonMax ] )
    //     {
    //         button[ ButtonMax ] ->setBitmap( m ? minmax_bits : max_bits );
    //         QToolTip::remove( button[ ButtonMax ] );
    //         QToolTip::add( button[ ButtonMax ], m ? i18n( "Restore" ) : i18n( "Maximize" ) );
    //     }
}

//////////////////////////////////////////////////////////////////////////////
// shadeChange()
// -------------
// Called when window shading changes

void Metal4kdeClient::shadeChange()
{
    //     bool s = isSetShade();
    //     if ( button[ ButtonShade ] )
    //     {
    //         button[ ButtonShade ] ->setBitmap( s ? shade_on_bits : shade_off_bits );
    //         QToolTip::remove( button[ ButtonShade ] );
    //         QToolTip::add( button[ ButtonShade ], s ? i18n( "Unshade" ) : i18n( "Shade" ) );
    //     }
}

//////////////////////////////////////////////////////////////////////////////
// keepAboveChange()
// ------------
// The above state has changed

void Metal4kdeClient::keepAboveChange( bool a )
{
    //     if ( button[ ButtonAbove ] )
    //     {
    //         button[ ButtonAbove ] ->setBitmap( a ? above_on_bits : above_off_bits );
    //     }
}

//////////////////////////////////////////////////////////////////////////////
// keepBelowChange()
// ------------
// The below state has changed

void Metal4kdeClient::keepBelowChange( bool b )
{
    //     if ( button[ ButtonBelow ] )
    //     {
    //         button[ ButtonBelow ] ->setBitmap( b ? below_on_bits : below_off_bits );
    //     }
}

//////////////////////////////////////////////////////////////////////////////
// borders()
// ----------
// Get the size of the borders

void Metal4kdeClient::borders( int &l, int &r, int &t, int &b ) const
{
    l = r = SIDEFRAMESIZE;
    t = TITLESIZE + FRAMESIZE;
    b = SIDEFRAMESIZE * 2;
}

//////////////////////////////////////////////////////////////////////////////
// resize()
// --------
// Called to resize the window

void Metal4kdeClient::resize( const QSize &size )
{
    widget() ->resize( size );
}

//////////////////////////////////////////////////////////////////////////////
// minimumSize()
// -------------
// Return the minimum allowable size for this window

QSize Metal4kdeClient::minimumSize() const
{
    return widget() ->minimumSize();
}

//////////////////////////////////////////////////////////////////////////////
// mousePosition()
// ---------------
// Return logical mouse position

KDecoration::Position Metal4kdeClient::mousePosition( const QPoint &point ) const
{
    const int corner = 24;
    Position pos;

    if ( point.y() <= FRAMESIZE )
    {
        // inside top frame
        if ( point.x() <= corner ) pos = PositionTopLeft;
        else if ( point.x() >= ( width() - corner ) ) pos = PositionTopRight;
        else pos = PositionTop;
    }
    else if ( point.y() >= ( height() - FRAMESIZE * 2 ) )
    {
        // inside handle
        if ( point.x() <= corner ) pos = PositionBottomLeft;
        else if ( point.x() >= ( width() - corner ) ) pos = PositionBottomRight;
        else pos = PositionBottom;
    }
    else if ( point.x() <= FRAMESIZE )
    {
        // on left frame
        if ( point.y() <= corner ) pos = PositionTopLeft;
        else if ( point.y() >= ( height() - corner ) ) pos = PositionBottomLeft;
        else pos = PositionLeft;
    }
    else if ( point.x() >= width() - FRAMESIZE )
    {
        // on right frame
        if ( point.y() <= corner ) pos = PositionTopRight;
        else if ( point.y() >= ( height() - corner ) ) pos = PositionBottomRight;
        else pos = PositionRight;
    }
    else
    {
        // inside the frame
        pos = PositionCenter;
    }
    return pos;
}

//////////////////////////////////////////////////////////////////////////////
// eventFilter()
// -------------
// Event filter

bool Metal4kdeClient::eventFilter( QObject *obj, QEvent *e )
{
    if ( obj != widget() ) return false;

    switch ( e->type() )
    {
    case QEvent::MouseButtonDblClick:
        {
            mouseDoubleClickEvent( static_cast<QMouseEvent *>( e ) );
            return true;
        }
    case QEvent::MouseButtonPress:
        {
            processMousePressEvent( static_cast<QMouseEvent *>( e ) );
            doShape();
            return true;
        }
    case QEvent::Paint:
        {
            paintEvent( static_cast<QPaintEvent *>( e ) );
            return true;
        }
    case QEvent::Resize:
        {
            resizeEvent( static_cast<QResizeEvent *>( e ) );
            return true;
        }
    case QEvent::Show:
        {
            showEvent( static_cast<QShowEvent *>( e ) );
            return true;
        }
    default:
        {
            return false;
        }
    }

    return false;
}

//////////////////////////////////////////////////////////////////////////////
// mouseDoubleClickEvent()
// -----------------------
// Doubleclick on title

void Metal4kdeClient::mouseDoubleClickEvent( QMouseEvent *e )
{
    if ( titlebar_->geometry().contains( e->pos() ) ) titlebarDblClickOperation();
}

//////////////////////////////////////////////////////////////////////////////
// paintEvent()
// ------------
// Repaint the window

void Metal4kdeClient::paintEvent( QPaintEvent* )
{
    if ( !Metal4kdeFactory::initialized() ) return ;

    QColorGroup group;
    QRect title( titlebar_->geometry() );
    QRect window( 0, 0, width(), TITLESIZE + FRAMESIZE );

    QPixmap pix( width(), TITLESIZE + FRAMESIZE );
    QPainter p;
    p.begin( &pix );
    p.drawTiledPixmap( window, Metal4kdeFactory::brushedMetalPix_ );
    p.drawPixmap( ( width() / 2 ) - 512 , 0, Metal4kdeFactory::brushedMetalLightningPix_, 0, 0, 1024, window.height() );
    //window.setHeight( 4 );
    p.drawTiledPixmap( 0, 0, width(), 4, Metal4kdeFactory::lightningPix_ );

    if ( Metal4kdeFactory::enableBubble_ )
    {
        if ( title.width() > 50 )
        {
            QFontMetrics fm( options() ->font( isActive(), false ) );
            int bubleLeft;
            int fontMargin = 10;
            int drawWidth = QMIN( title.width(), fm.width( caption() ) + ( 2 * fontMargin ) );
            switch ( Metal4kdeFactory::titleAlign() )
            {
            case Qt::AlignLeft:
                {
                    bubleLeft = title.x();
                    break;
                }
            case Qt::AlignHCenter:
                {
                    bubleLeft = title.x() + ( title.width() / 2 ) - ( drawWidth / 2 ) ;
                    break;
                }
            case Qt::AlignRight:
                {
                    bubleLeft = title.right() - drawWidth ;
                    break;
                }
            default:
                {
                    bubleLeft = 0 ;
                    break;
                }
            }

            p.drawPixmap( bubleLeft, 3, Metal4kdeFactory::bubbleBevelDownLPix_ );
            p.drawTiledPixmap( bubleLeft + 15, 3, drawWidth - 30, 27 , Metal4kdeFactory::bubbleBevelDownMPix_ );
            p.drawPixmap( bubleLeft + drawWidth - 15 , 3, Metal4kdeFactory::bubbleBevelDownRPix_ );

            p.drawPixmap( bubleLeft, 3, isActive() ? Metal4kdeFactory::bubbleLPix_ : Metal4kdeFactory::bubbleInactiveLPix_ );
            p.drawTiledPixmap( bubleLeft + 15, 3, drawWidth - 30, 27 , isActive() ? Metal4kdeFactory::bubbleMPix_ : Metal4kdeFactory::bubbleInactiveMPix_ );
            p.drawPixmap( bubleLeft + drawWidth - 15 , 3, isActive() ? Metal4kdeFactory::bubbleRPix_ : Metal4kdeFactory::bubbleInactiveRPix_ );
        }
    }

    p.end();

    QPainter painter( widget() );

    // draw the titlebar
    //painter.drawTiledPixmap( window, Metal4kdeFactory::brushedMetalPix_ );
    painter.drawTiledPixmap( window, pix );
    //painter.drawTiledPixmap( window, Metal4kdeFactory::brushedMetalPix_ );
    //painter.drawPixmap( ( width() / 2 ) - 512 , 0, Metal4kdeFactory::brushedMetalLightningPix_, 0, 0, 1024, window.height() );
    //window.setHeight( 4 );
    //painter.drawTiledPixmap( window, Metal4kdeFactory::lightningPix_ );

    //     if ( Metal4kdeFactory::enableBubble_ )
    //     {
    //         if ( title.width() > 50 )
    //         {
    //             QFontMetrics fm( options() ->font( isActive(), false ) );
    //             int bubleLeft;
    //             int fontMargin = 10;
    //             int drawWidth = QMIN( title.width(), fm.width( caption() ) + ( 2 * fontMargin ) );
    //             switch ( Metal4kdeFactory::titleAlign() )
    //             {
    //             case Qt::AlignLeft:
    //                 {
    //                     bubleLeft = title.x();
    //                     break;
    //                 }
    //             case Qt::AlignHCenter:
    //                 {
    //                     bubleLeft = title.x() + ( title.width() / 2 ) - ( drawWidth / 2 ) ;
    //                     break;
    //                 }
    //             case Qt::AlignRight:
    //                 {
    //                     bubleLeft = title.right() - drawWidth ;
    //                     break;
    //                 }
    //             default:
    //                 {
    //                     bubleLeft = 0 ;
    //                     break;
    //                 }
    //             }
    //
    // //             painter.drawPixmap( bubleLeft, 3, Metal4kdeFactory::bubbleBevelDownLPix_ );
    // //             painter.drawTiledPixmap( bubleLeft + 15, 3, drawWidth - 30, 27 , Metal4kdeFactory::bubbleBevelDownMPix_ );
    // //             painter.drawPixmap( bubleLeft + drawWidth - 15 , 3, Metal4kdeFactory::bubbleBevelDownRPix_ );
    //
    //             //painter.drawPixmap( bubleLeft, 3, isActive() ? Metal4kdeFactory::bubbleLPix_ : Metal4kdeFactory::bubbleInactiveLPix_);
    //             painter.drawTiledPixmap( bubleLeft + 15, 3, drawWidth - 30, 27 ,isActive() ? Metal4kdeFactory::bubbleMPix_ : Metal4kdeFactory::bubbleInactiveMPix_ );
    //             //painter.drawPixmap( bubleLeft + drawWidth - 15 , 3,isActive() ? Metal4kdeFactory::bubbleRPix_ : Metal4kdeFactory::bubbleInactiveRPix_ );
    //         }
    //     }

    // draw title text
    painter.setFont( options() ->font( isActive(), false ) );

    if ( Metal4kdeFactory::enableFontShadow_ )
    {
        painter.setPen( Qt::gray );
        painter.drawText( title.x() + 2 + 10,
                          title.y() + 2,
                          title.width() - 10 * 2,
                          title.height(),
                          Metal4kdeFactory::titleAlign() | AlignVCenter,
                          caption() );
    }

    painter.setPen( options() ->color( KDecoration::ColorFont, isActive() ) );
    painter.drawText( title.x() + 10,
                      title.y(),
                      title.width() - 10 * 2,
                      title.height(),
                      Metal4kdeFactory::titleAlign() | AlignVCenter,
                      caption() );


    // draw frame
    //group = options() ->colorGroup( KDecoration::ColorFrame, isActive() );

    QRect frame( 0, 0, width(), SIDEFRAMESIZE );
    //painter.fillRect(frame, Qt::green);
    // left
    frame.setRect( 0, 0, SIDEFRAMESIZE, height() );
    painter.drawTiledPixmap( frame , Metal4kdeFactory::frameLPix_ );
    // right
    frame.setRect( width() - SIDEFRAMESIZE, 0, SIDEFRAMESIZE, height() );
    painter.drawTiledPixmap( frame , Metal4kdeFactory::frameRPix_ );
    // buttom
    frame.setRect( 0, height() - SIDEFRAMESIZE * 2, width(), SIDEFRAMESIZE * 2 );
    painter.drawTiledPixmap( frame, Metal4kdeFactory::brushedMetalPix_ );
    painter.drawPixmap( ( width() / 2 ) - 512 , frame.y(), Metal4kdeFactory::brushedMetalLightningPix_, 0, 94, 1024, frame.height() );
    painter.drawTiledPixmap( 0, height() - 4, width(), 4, Metal4kdeFactory::shadingPix_ );
}

// void Metal4kdeClient::paintEvent(QPaintEvent*)
// {
//     if (!Metal4kdeFactory::initialized()) return;
//
//     QColorGroup group;
//     QPainter painter(widget());
//
//     // draw the titlebar
//     QRect title(titlebar_->geometry());
//     group = options()->colorGroup(KDecoration::ColorTitleBar, isActive());
//     painter.fillRect(title, group.background());
//     painter.setPen(group.dark());
//     painter.drawRect(title);
//
//     // draw title text
//     painter.setFont(options()->font(isActive(), false));
//     painter.setPen(options()->color(KDecoration::ColorFont, isActive()));
//     painter.drawText(title.x() + FRAMESIZE, title.y(),
//                      title.width() - FRAMESIZE * 2, title.height(),
//                      Metal4kdeFactory::titleAlign() | AlignVCenter,
//                      caption());
//
//     // draw frame
//     group = options()->colorGroup(KDecoration::ColorFrame, isActive());
//
//     QRect frame(0, 0, width(), FRAMESIZE);
//     painter.fillRect(frame, group.background());
//     frame.setRect(0, 0, FRAMESIZE, height());
//     painter.fillRect(frame, group.background());
//     frame.setRect(0, height() - FRAMESIZE*2, width(), FRAMESIZE*2);
//     painter.fillRect(frame, group.background());
//     frame.setRect(width()-FRAMESIZE, 0, FRAMESIZE, height());
//     painter.fillRect(frame, group.background());
//
//     // outline the frame
//     painter.setPen(group.dark());
//     frame = widget()->rect();
//     painter.drawRect(frame);
//     frame.setRect(frame.x() + FRAMESIZE-1, frame.y() + FRAMESIZE-1,
//                   frame.width() - FRAMESIZE*2 +2,
//                   frame.height() - FRAMESIZE*3 +2);
//     painter.drawRect(frame);
// }
//






//////////////////////////////////////////////////////////////////////////////
// resizeEvent()
// -------------
// Window is being resized

void Metal4kdeClient::resizeEvent( QResizeEvent * )
{
    if ( widget() ->isShown() )
    {
        QRegion region = widget() ->rect();
        //region = region.subtract( titlebar_->geometry() );
        widget() ->erase( region );
    }
    for ( int n = 0; n < ButtonTypeCount; n++ )
        if ( button[ n ] ) button[ n ] ->reset();
    widget() ->repaint( false );
    doShape();
}

//////////////////////////////////////////////////////////////////////////////
// showEvent()
// -----------
// Window is being shown

void Metal4kdeClient::showEvent( QShowEvent * )
{
    widget() ->repaint();
    doShape();
}

//////////////////////////////////////////////////////////////////////////////
// maxButtonPressed()
// -----------------
// Max button was pressed

void Metal4kdeClient::maxButtonPressed()
{
    if ( button[ ButtonMax ] )
    {
#if KDE_IS_VERSION(3, 3, 0)
        maximize( button[ ButtonMax ] ->lastMousePress() );
#else
        switch ( button[ ButtonMax ] ->lastMousePress() )
        {
        case MidButton:
            maximize( maximizeMode() ^ MaximizeVertical );
            break;
        case RightButton:
            maximize( maximizeMode() ^ MaximizeHorizontal );
            break;
        default:
            ( maximizeMode() == MaximizeFull ) ? maximize( MaximizeRestore )
            : maximize( MaximizeFull );
        }
#endif

    }
}

//////////////////////////////////////////////////////////////////////////////
// shadeButtonPressed()
// -----------------
// Shade button was pressed

void Metal4kdeClient::shadeButtonPressed()
{
    if ( button[ ButtonShade ] )
    {
        setShade( !isSetShade() );
    }
}

//////////////////////////////////////////////////////////////////////////////
// aboveButtonPressed()
// -----------------
// Above button was pressed

void Metal4kdeClient::aboveButtonPressed()
{
    if ( button[ ButtonAbove ] )
    {
        setKeepAbove( !keepAbove() );
    }
}

//////////////////////////////////////////////////////////////////////////////
// belowButtonPressed()
// -----------------
// Below button was pressed

void Metal4kdeClient::belowButtonPressed()
{
    if ( button[ ButtonBelow ] )
    {
        setKeepBelow( !keepBelow() );
    }
}

//////////////////////////////////////////////////////////////////////////////
// menuButtonPressed()
// -------------------
// Menu button was pressed (popup the menu)

void Metal4kdeClient::menuButtonPressed()
{
    if ( button[ ButtonMenu ] )
    {
        QPoint p( button[ ButtonMenu ] ->rect().bottomLeft().x(),
                  button[ ButtonMenu ] ->rect().bottomLeft().y() );
        KDecorationFactory* f = factory();
        showWindowMenu( button[ ButtonMenu ] ->mapToGlobal( p ) );
        if ( !f->exists( this ) ) return ; // decoration was destroyed
        button[ ButtonMenu ] ->setDown( false );
    }
}

#include "Metal4kdeclient.moc"
