/***************************************************************************
 *   Copyright (C) 2004 by moty rahamim                                    *
 *   moty@onexige.com                                                      *
 *                                                                         *
 *   just another metalic like Kstyle & Kwin, this theme is based on:      *
 *   Phase & example by David Johnson                                      *
 *   liquid by Daniel M. Duley a.k.a Mosfet <mosfet@idunno>                *
 *   baghira by Thomas Lbking <thomas.luebking@web.de>                    * 
 *                                                                         *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#ifndef Metal4kdeCLIENT_H
#define Metal4kdeCLIENT_H

#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>
#include <qpixmap.h>


class QSpacerItem;
class QPoint;

namespace Metal4kde
{

    class Metal4kdeClient;

    enum ButtonType {
        ButtonHelp = 0,
        ButtonMax,
        ButtonMin,
        ButtonClose,
        ButtonMenu,
        ButtonSticky,
        ButtonAbove,
        ButtonBelow,
        ButtonShade,
        ButtonTypeCount
    };

    // Metal4kdeFactory /////////////////////////////////////////////////////////////

    class Metal4kdeFactory: public KDecorationFactory
    {
    public:
        Metal4kdeFactory();
        virtual ~Metal4kdeFactory();
        virtual KDecoration *createDecoration( KDecorationBridge *b );
        virtual bool reset( unsigned long changed );

        static bool initialized();
        static Qt::AlignmentFlags titleAlign();

    private:
        bool readConfig();
        void createPixmaps();
        QPixmap* adjustHSV( QImage &img, const QColor &c, bool blend = TRUE ) const;
        QImage* adjustHSVImage( QImage &img, const QColor &c, bool blend ) const;

    public:
        static bool enableFontShadow_;
        static bool enableBubble_;
    private:
        static bool initialized_;
        static Qt::AlignmentFlags titlealign_;
        QWMatrix iMatrix_;
        QColor buttonHelpCol_, buttonMaxCol_, buttonMinCol_, buttonCloseCol_, buttonMenuCol_, buttonStikyCol_, buttonAboveCol_, buttonBelowCol_, buttonShadeCol_, bubbleCol_, bubbleInactiveCol_, buttonInactiveCol_, brushedMetalCol_;

    public:
        static QPixmap brushedMetalPix_, brushedMetalLightningPix_, bubbleLPix_, bubbleMPix_, bubbleRPix_, bubbleInactiveLPix_, bubbleInactiveMPix_, bubbleInactiveRPix_,bubbleBevelDownLPix_, bubbleBevelDownMPix_, bubbleBevelDownRPix_, frameLPix_, frameRPix_, lightningPix_, shadingPix_, buttonPix_, buttonBevelDownPix_, buttonBevelUpPix_, buttonHelpPix_, buttonHelpHoverPix_, buttonHelpInactivePix_, buttonHelpBevelPix_, buttonMaxPix_, buttonMaxHoverPix_, buttonMinPix_, buttonMinHoverPix_, buttonClosePix_, buttonCloseHoverPix_, buttonMenuPix_, buttonMenuHoverPix_, buttonStickyPix_, buttonStickyHoverPix_, buttonAbovePix_, buttonAboveHoverPix_, buttonBelowPix_, buttonBelowHoverPix_, buttonShadePix_, buttonShadeHoverPix_, buttonInactivePix_, buttonInactiveHoverPix_;
    };

    inline bool Metal4kdeFactory::initialized()
    { return initialized_; }

    inline Qt::AlignmentFlags Metal4kdeFactory::titleAlign()
    { return titlealign_; }









    // Metal4kdeButton //////////////////////////////////////////////////////////////

    class Metal4kdeButton : public QButton
    {
    public:
        Metal4kdeButton( Metal4kdeClient *parent = 0, const char *name = 0,
                       const QString &tip = NULL,
                       ButtonType type = ButtonHelp, QPixmap *pix = 0, QPixmap *pixHover = 0 );
        ~Metal4kdeButton();

        void setPixmap( QPixmap *pix, QPixmap *pixHover );
        QSize sizeHint() const;
        ButtonState lastMousePress() const;
        void reset();

    private:
        void enterEvent( QEvent *e );
        void leaveEvent( QEvent *e );
        void mousePressEvent( QMouseEvent *e );
        void mouseReleaseEvent( QMouseEvent *e );
        void drawButton( QPainter *painter );

    private:
        Metal4kdeClient *client_;
        ButtonType type_;
        QPixmap *pix_, *pixHover_;
        ButtonState lastmouse_;
        Metal4kdeClient *parent_;
        bool hover_;
    };

    inline Qt::ButtonState Metal4kdeButton::lastMousePress() const
        { return lastmouse_; }

    inline void Metal4kdeButton::reset()
    { repaint( false ); }








    // Metal4kdeClient //////////////////////////////////////////////////////////////

    class Metal4kdeClient : public KDecoration
    {
        Q_OBJECT
    public:
        Metal4kdeClient( KDecorationBridge *b, KDecorationFactory *f );
        virtual ~Metal4kdeClient();

        virtual void init();

        virtual void activeChange();
        virtual void desktopChange();
        virtual void captionChange();
        virtual void iconChange();
        virtual void maximizeChange();
        virtual void shadeChange();

        virtual void borders( int &l, int &r, int &t, int &b ) const;
        virtual void resize( const QSize &size );
        virtual QSize minimumSize() const;
        virtual Position mousePosition( const QPoint &point ) const;

    private:
        void addButtons( QBoxLayout* layout, const QString& buttons );
        void doShape();

        bool eventFilter( QObject *obj, QEvent *e );
        void mouseDoubleClickEvent( QMouseEvent *e );
        void paintEvent( QPaintEvent *e );
        void resizeEvent( QResizeEvent * );
        void showEvent( QShowEvent * );

    private slots:
        void maxButtonPressed();
        void menuButtonPressed();
        void aboveButtonPressed();
        void belowButtonPressed();
        void shadeButtonPressed();
        void keepAboveChange( bool );
        void keepBelowChange( bool );

    private:
        Metal4kdeButton *button[ ButtonTypeCount ];
        QSpacerItem *titlebar_;

        Metal4kdeFactory *decoFactory_;
    };

} // namespace Metal4kde

#endif // Metal4kdeCLIENT_H
