/***************************************************************************
*   Copyright (C) 2004 by moty rahamim                                    *
*   moty@onexige.com                                                      *
*                                                                         *
*   just another metalic like Kstyle & Kwin, this theme is based on:      *
*   Phase & example by David Johnson                                      *
*   liquid by Daniel M. Duley a.k.a Mosfet <mosfet@idunno>                *
*   baghira by Thomas Lbking <thomas.luebking@web.de>                    * 
*                                                                         *
*                                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <kconfig.h>
#include <klocale.h>
#include <kglobal.h>
#include <qbuttongroup.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qwhatsthis.h>
#include <kcolorbutton.h>
#include <qcheckbox.h>

#include "Metal4kdeconfig.h"
#include "configdialog.h"

//////////////////////////////////////////////////////////////////////////////
// Metal4kdeConfig()
// -------------
// Constructor

Metal4kdeConfig::Metal4kdeConfig( KConfig* config, QWidget* parent )
        : QObject( parent ), config_( 0 ), dialog_( 0 )
{
    // create the configuration object
    config_ = new KConfig( "kwinMetal4kderc" );
    KGlobal::locale() ->insertCatalogue( "kwin_Metal4kde_config" );

    // create and show the configuration dialog
    dialog_ = new ConfigDialog( parent );
    dialog_->show();

    // load the configuration
    load( config_ );

    // setup the connections
    connect( dialog_->titlealign, SIGNAL( clicked( int ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->EnableFontsShadowCB, SIGNAL( toggled( bool ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->EnableBubbleCB, SIGNAL( toggled( bool ) ), this, SLOT( selectionChanged( ) ) );


    connect( dialog_->kcbButtonHelp, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonMax, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonMin, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonClose, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonStiky, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonAbove, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonBelow, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtonShade, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbBubble, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbBubbleInactive, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbButtunInactive, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );
    connect( dialog_->kcbBrushedMetal, SIGNAL( changed( const QColor & ) ), this, SLOT( selectionChanged( ) ) );


}

//////////////////////////////////////////////////////////////////////////////
// ~Metal4kdeConfig()
// --------------
// Destructor

Metal4kdeConfig::~Metal4kdeConfig()
{
    if ( dialog_ ) delete dialog_;
    if ( config_ ) delete config_;
}

//////////////////////////////////////////////////////////////////////////////
// selectionChanged()
// ------------------
// Selection has changed

void Metal4kdeConfig::selectionChanged( )
{
    emit changed();
}

//////////////////////////////////////////////////////////////////////////////
// load()
// ------
// Load configuration data

void Metal4kdeConfig::load( KConfig* )
{
    config_->setGroup( "General" );

    QString value = config_->readEntry( "TitleAlignment", "AlignHCenter" );
    QRadioButton *button = ( QRadioButton* ) dialog_->titlealign->child( value );
    if ( button ) button->setChecked( true );
    dialog_->EnableFontsShadowCB->setChecked( config_->readBoolEntry( "EnableFontsShadow", FALSE ) );
    dialog_->EnableBubbleCB->setChecked( config_->readBoolEntry( "EnableBubble", FALSE ) );

    dialog_->kcbButtonHelp->setColor( QColor( config_->readNumEntry( "ButtonHelp", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonMax->setColor( QColor( config_->readNumEntry( "ButtonMax", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonMin->setColor( QColor( config_->readNumEntry( "ButtonMin", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonClose->setColor( QColor( config_->readNumEntry( "ButtonClose", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonStiky->setColor( QColor( config_->readNumEntry( "ButtonStiky", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonAbove->setColor( QColor( config_->readNumEntry( "ButtonAbove", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonBelow->setColor( QColor( config_->readNumEntry( "ButtonBelow", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtonShade->setColor( QColor( config_->readNumEntry( "ButtonShade", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbBubble->setColor( QColor( config_->readNumEntry( "Bubble", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbBubbleInactive->setColor( QColor( config_->readNumEntry( "BubbleInactive", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbButtunInactive->setColor( QColor( config_->readNumEntry( "ButtunInactive", ( int ) Qt::gray.rgb() ) ) );
    dialog_->kcbBrushedMetal->setColor( QColor( config_->readNumEntry( "BrushedMetal", ( int ) Qt::gray.rgb() ) ) );
}

//////////////////////////////////////////////////////////////////////////////
// save()
// ------
// Save configuration data

void Metal4kdeConfig::save( KConfig* )
{
    config_->setGroup( "General" );

    QRadioButton *button = ( QRadioButton* ) dialog_->titlealign->selected();
    if ( button ) config_->writeEntry( "TitleAlignment", QString( button->name() ) );
    config_->writeEntry( "EnableFontsShadow", dialog_->EnableFontsShadowCB->isChecked() );
    config_->writeEntry( "EnableBubble", dialog_->EnableBubbleCB->isChecked() );

    config_->writeEntry( "ButtonHelp", ( int ) dialog_->kcbButtonHelp->color().rgb() );
    config_->writeEntry( "ButtonMax", ( int ) dialog_->kcbButtonMax->color().rgb() );
    config_->writeEntry( "ButtonMin", ( int ) dialog_->kcbButtonMin->color().rgb() );
    config_->writeEntry( "ButtonClose", ( int ) dialog_->kcbButtonClose->color().rgb() );
    config_->writeEntry( "ButtonStiky", ( int ) dialog_->kcbButtonStiky->color().rgb() );
    config_->writeEntry( "ButtonAbove", ( int ) dialog_->kcbButtonAbove->color().rgb() );
    config_->writeEntry( "ButtonBelow", ( int ) dialog_->kcbButtonBelow->color().rgb() );
    config_->writeEntry( "ButtonShade", ( int ) dialog_->kcbButtonShade->color().rgb() );
    config_->writeEntry( "Bubble", ( int ) dialog_->kcbBubble->color().rgb() );
    config_->writeEntry( "BubbleInactive", ( int ) dialog_->kcbBubbleInactive->color().rgb() );
    config_->writeEntry( "ButtunInactive", ( int ) dialog_->kcbButtunInactive->color().rgb() );
    config_->writeEntry( "BrushedMetal", ( int ) dialog_->kcbBrushedMetal->color().rgb() );

    config_->sync();
}

//////////////////////////////////////////////////////////////////////////////
// defaults()
// ----------
// Set configuration defaults

void Metal4kdeConfig::defaults()
{
    QRadioButton * button =
        ( QRadioButton* ) dialog_->titlealign->child( "AlignHCenter" );
    if ( button ) button->setChecked( true );
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

extern "C"
{
    QObject* allocate_config( KConfig * config, QWidget * parent )
    {
        return ( new Metal4kdeConfig( config, parent ) );
    }
}

#include "Metal4kdeconfig.moc"
